/************************************************************************/
/*                                                                      */
/*             ,     ..        */
/*                   ,   ,   .               */
/*                                                                      */
/************************************************************************/

#pragma once

#include <stack>
#include <map>
#include "program.h"

#define MAX_DATASTACK_DEEP		1024
#define MAX_CODESTACK_DEEP		1024

//     
//   

PVarVal Add(PVarVal& x, PVarVal& y);
PVarVal Sub(PVarVal& x, PVarVal& y);
PVarVal Mul(PVarVal& x, PVarVal& y);
PVarVal Div(PVarVal& x, PVarVal& y);
PVarVal Neg(PVarVal& x);
PVarVal And(PVarVal& x, PVarVal& y);
PVarVal Or(PVarVal& x, PVarVal& y);
PVarVal Xor(PVarVal& x, PVarVal& y);
PVarVal Not(PVarVal& x);
PVarVal Eq(PVarVal& x, PVarVal& y);
PVarVal NotEq(PVarVal& x, PVarVal& y);
PVarVal Less(PVarVal& x, PVarVal& y);
PVarVal LessEq(PVarVal& x, PVarVal& y);
PVarVal Greater(PVarVal& x, PVarVal& y);
PVarVal GreaterEq(PVarVal& x, PVarVal& y);
bool UnTypecast(PVarVal& v, eVarType Type);
bool BinTypecast(PVarVal& x, PVarVal& y);

/************************************************************************/
/*                                         */
/************************************************************************/

class CInterpreter
{
private:
	//    -  
	//   ,   (, ) 
	//   ,  .  .
	struct StackFrame
	{
		eStmtType			Type;
		PFunction*			pFn;
		STMTLIST::iterator	NextStmt;
		STMTLIST::iterator	End;
	};

	typedef vector<PVarVal> VARARGS;
	typedef PVarVal (CInterpreter::*INTPROC)(VARARGS& args);

	//      
	struct IntProc
	{
		const char* pszName;		// 
		bool		bLValueArgs;	//     
		bool		bVarArgs;		// -  .  
		int			nArgCount;		// -  
		INTPROC		pFn;			//   
	};

	typedef std::map<string,IntProc> INTPROCSMAP;

	Program*				m_pProg;
	PStatement*				m_pStmt;
	std::stack<PVarVal>		m_DataStack;
	std::stack<StackFrame>	m_CodeStack;
	INTPROCSMAP				m_IntProcs;
	static IntProc			g_IntProcs[];

	//    
	void InitIntProcs();
	//         
	void Error(const char* fmt, ...);
	//  r-,   
	PVarVal PopRVal(eVarType Type = vtEmpty);
	//  l-,   
	PVarVal PopLVal();
	//     
	void PushSF(eStmtType Type, STMTLIST& statements, PFunction* pFn = NULL);
	//          
	void PopSF();
	//      
	void FindFrameType(eStmtType Type, const char* err);
	//  
	void UnOp(PVarVal (*OpFn)(PVarVal& x), const char* name);
	//  
	void BinOp(PVarVal (*OpFn)(PVarVal& x, PVarVal& y), const char* name);

	//    
	PVarVal ExprVal(PExpr& expr, eVarType Type = vtEmpty)
	{
		RunExpr(expr);
		return PopRVal(Type);
	}
	
	//    
	void NextStmt()
	{
		++m_CodeStack.top().NextStmt;
	}

	//    
	template <class T>
	void PushC(eVarType Type, T val)
	{
		PVarVal v(val);
		v.Type = Type;
		m_DataStack.push(v);
	}

	//    
	void RunStatement(PStatement* stmt);
	//  if
	void RunIfStmt(PIfStmt* stmt);
	//  while
	void RunWhileStmt(PWhileStmt* stmt);
	//  break
	void RunBreakStmt();
	//  continue
	void RunContinueStmt();
	//  return
	void RunReturnStmt(PReturnStmt* stmt);
	//  ''
	void RunExprStmt(PExprStmt* stmt);
	// 
	void RunExpr(PExpr& expr);
	//  
	void RunCallSub(const char* pszFnName);

	//   
	PVarVal UnMathFn(PVarVal& v, const char* name, double (*fn)(double));

	//    
	PVarVal IntProc_Print(VARARGS& args);
	PVarVal IntProc_Input(VARARGS& args);
	PVarVal IntProc_ToStr(VARARGS& args);
	PVarVal IntProc_Sqrt(VARARGS& args);
	PVarVal IntProc_Pow(VARARGS& args);
	PVarVal IntProc_Sin(VARARGS& args);
	PVarVal IntProc_Fabs(VARARGS& args);

public:
	CInterpreter(Program* pProg);
	//   
	bool DoStep();
};
