/************************************************************************/
/*                                                                      */
/*             ,     ..        */
/*                   ,   ,   .               */
/*                                                                      */
/************************************************************************/

#pragma warning(disable:4786)

#include <math.h>
#include <stdarg.h>
#include "interp.h"
#include "error.h"

//    
CInterpreter::IntProc CInterpreter::g_IntProcs[] = 
{
	{"print",	false,	true,	0,	IntProc_Print},
	{"input",	true,	true,	0,	IntProc_Input},
	{"tostr",	false,	false,	1,	IntProc_ToStr},
	{"sqrt",	false,	false,	1,	IntProc_Sqrt},
	{"pow",		false,	false,	2,	IntProc_Pow},
	{"sin",		false,	false,	1,	IntProc_Sin},
	{"fabs",	false,	false,	1,	IntProc_Fabs}
};

//    
void CInterpreter::InitIntProcs()
{
	for (int i = 0; i < sizeof(g_IntProcs) / sizeof(g_IntProcs[0]); i++)
	{
		m_IntProcs[g_IntProcs[i].pszName] = g_IntProcs[i];
	}
}

//   "print".
//      .
PVarVal CInterpreter::IntProc_Print(VARARGS& args)
{
	VARARGS::iterator it = args.begin();
	for (; it != args.end(); ++it)
	{
		PVarVal& v = (*it);
		switch (v.Type)
		{
			case vtInt:		printf("%I64i", v.iVal); break;
			case vtFloat:	printf("%lf", v.fltVal); break;
			case vtString:	printf("%s", v.strVal); break;
			case vtBool:	printf("%s", v.bVal ? "true" : "false"); break;
		}
	}
	return PVarVal();
}

//   "input".
//        .
PVarVal CInterpreter::IntProc_Input(VARARGS& args)
{
	VARARGS::iterator it = args.begin();
	int nValues = 0;
	for (; it != args.end(); ++it)
	{
		PVarVal& v = m_CodeStack.top().pFn->Vars[(*it).strVal];
		switch (v.Type)
		{
			case vtInt:		nValues += scanf("%I64i", &v.iVal); break;
			case vtFloat:	nValues += scanf("%lf", &v.fltVal); break;
			case vtString:	
				{
					char szBuf[4096] = {0};
					gets(szBuf);
					szBuf[strlen(szBuf)] = '\0';
					nValues++;
					v = PVarVal(szBuf);
				}
		}
	}
	return PVarVal((__int64)nValues);
}

//   "print".
//    .
PVarVal CInterpreter::IntProc_ToStr(VARARGS& args)
{
	char szBuf[256] = {0};
	PVarVal& v = args[0];
	switch (v.Type)
	{
		case vtInt:		sprintf(szBuf, "%I64i", v.iVal); break;
		case vtFloat:	sprintf(szBuf, "%lf", v.fltVal); break;
		case vtString:	sprintf(szBuf, "%s", v.strVal); break;
		case vtBool:	sprintf(szBuf, "%s", v.bVal ? "true" : "false"); break;
	}
	return PVarVal(szBuf);
}

//   
PVarVal CInterpreter::UnMathFn(PVarVal& v, const char* name, double (*fn)(double))
{
	try
	{
		if (!UnTypecast(v, vtFloat))
		{
			Error(E_INV_TYPEOP, v.Type, name);
		}
		return fn(v);
	}
	catch (...)
	{
		Error(E_MATH_ERR, name);
	}
	return PVarVal();
}

//   "print".
//     .
PVarVal CInterpreter::IntProc_Sqrt(VARARGS& args)
{
	return UnMathFn(args[0], "sqrt", ::sqrt);
}

//   "print".
//  x^y.
PVarVal CInterpreter::IntProc_Pow(VARARGS& args)
{
	try
	{
		if (!UnTypecast(args[0], vtFloat))
		{
			Error(E_INV_TYPEOP, args[0].Type, "pow");
		}
		if (!UnTypecast(args[1], vtFloat))
		{
			Error(E_INV_TYPEOP, args[1].Type, "pow");
		}
		return ::pow(args[0], args[1]);
	}
	catch (...)
	{
		Error(E_MATH_ERR, "pow");
	}
	return PVarVal();
}

//   "print".
//    .
PVarVal CInterpreter::IntProc_Sin(VARARGS& args)
{
	return UnMathFn(args[0], "sin", ::sin);
}

//   "print".
//    .
PVarVal CInterpreter::IntProc_Fabs(VARARGS& args)
{
	return UnMathFn(args[0], "fabs", ::fabs);
}
