/************************************************************************/
/*                                                                      */
/*             ,     ..        */
/*                   ,   ,   .               */
/*                                                                      */
/************************************************************************/

#pragma once

#include "lexer.h"

/************************************************************************/
/*                                       */
/************************************************************************/

class CParser
{
private:
	Program*	m_pProg;
	CLexer*		m_pLexer;
	CToken		m_tkLast;
	bool		m_bUnget;

	//      
	CToken& get()
	{
		if (m_bUnget)
		{
			m_bUnget = false;
			return m_tkLast;
		}
		return m_tkLast = m_pLexer->GetToken();
	}
	//      
	void unget()
	{
		m_bUnget = true;
	}

	//         
	void Error(const char* fmt, ...);
	//       
	CToken& Token(eTokenType type);
	//  
	CToken& Ident()
	{
		return Token(ttIdent);
	}

	//    

	//  
	void FnList(FNMAP& functions);
	//  
	void StmtList(STMTLIST& statements);

	//  if
	void IfStmt(PIfStmt& stmt);
	//  while
	void WhileStmt(PWhileStmt& stmt);
	//  break
	void BreakStmt(PBreakStmt& stmt);
	//  continue
	void ContinueStmt(PContinueStmt& stmt);
	//  return
	void ReturnStmt(PReturnStmt& stmt);
	//  ''
	void ExprStmt(PExprStmt& stmt);
	// 
	void Expr(PExpr& expr);

	//    

	// 
	void OpAssign(OPLIST& op);
	//  ""
	void OpOr(OPLIST& op);
	//  " "
	void OpXor(OPLIST& op);
	//  ""
	void OpAnd(OPLIST& op);
	//  -  
	void OpEq(OPLIST& op);
	// ,   ..
	void OpCmp(OPLIST& op);
	//  - 
	void OpAdd(OPLIST& op);
	//  - 
	void OpMul(OPLIST& op);
	//  
	void OpNeg(OPLIST& op);
	//  ""
	void OpNot(OPLIST& op);
	//  - ,      
	void OpOperand(OPLIST& op);
	//  
	void OpCallSub(OPLIST& op, const char* pszName);

public:
	CParser(Program* pProg, CLexer* pLexer);
	//    
	void Parse();
};
