/************************************************************************/
/*                                                                      */
/*             ,     ..        */
/*                   ,   ,   .               */
/*                                                                      */
/************************************************************************/

#pragma once

//   
enum eTokenType
{
	ttError,
	ttEOF,
	ttIntConst,
	ttFloatConst,
	ttStrConst,
	ttBoolConst,
	ttIdent,
	ttKwTrue,		// true
	ttKwFalse,		// false
	ttKwFunction,	// function
	ttKwIf,			// if
	ttKwWhile,		// while
	ttKwBreak,		// break
	ttKwContinue,	// continue
	ttKwReturn,		// return
	ttOpAssign,		// =
	ttOpPlus,		// +
	ttOpMinus,		// -
	ttOpMul,		// *
	ttOpDiv,		// /
	ttOpEqual,		// ==
	ttOpNotEq,		// !=
	ttOpLess,		// <
	ttOpLessEq,		// <=
	ttOpGreater,	// >
	ttOpGreaterEq,	// >=
	ttOpAnd,		// &&
	ttOpOr,			// ||
	ttOpXor,		// ^^
	ttOpNot,		// !
	ttDlmRBracket,	// (
	ttDlmLBracket,	// )
	ttDlmBegBlock,	// {
	ttDlmEndBlock,	// }
	ttDlmComma,		// ,
	ttDlmSemicolon	// ;
};

//  
struct CToken
{
	eTokenType	Type;		// 
	int			nLine;		// 
	int			nCol;		// 
	
	//  (   )
	union
	{
		__int64		iVal;
		double		fltVal;
		const char* strVal;
		bool		bVal;
	};

	// 
	CToken() : Type(ttError), nLine(0), nCol(0)
	{
	}
	CToken(eTokenType _Type, int _nLine, int _nCol) : 
		Type(_Type), nLine(_nLine), nCol(_nCol)
	{
	}
};

//        
inline const char* TokenTypeToStr(eTokenType type)
{
	switch (type)
	{
		case ttEOF:			return "<EOF>";
		case ttIntConst:	return "integer constant";
		case ttFloatConst:	return "float constant";
		case ttStrConst:	return "string constant";
		case ttIdent:		return "identifier";
		case ttKwFunction:	return "'function'";
		case ttKwIf:		return "'if'";
		case ttKwWhile:		return "'while'";
		case ttKwBreak:		return "'break'";
		case ttKwContinue:	return "'continue'";
		case ttKwReturn:	return "'return'";
		case ttOpAssign:	return "'='";
		case ttOpPlus:		return "'+'";
		case ttOpMinus:		return "'-'";
		case ttOpMul:		return "'*'";
		case ttOpDiv:		return "'/'";
		case ttOpEqual:		return "'=='";
		case ttOpNotEq:		return "'!='";
		case ttOpLess:		return "'<'";
		case ttOpLessEq:	return "'<='";
		case ttOpGreater:	return "'>'";
		case ttOpGreaterEq:	return "'>='";
		case ttOpAnd:		return "'&&'";
		case ttOpOr:		return "'||'";
		case ttOpXor:		return "'^^'";
		case ttOpNot:		return "'!'";
		case ttDlmRBracket:	return "'('";
		case ttDlmLBracket:	return "')'";
		case ttDlmBegBlock:	return "'{'";
		case ttDlmEndBlock:	return "'}'";
		case ttDlmComma:	return "','";
		case ttDlmSemicolon:return "';'";
	}

	return "";
}
